<?php

/**
 * This class is a common class for every plugin developed by orbisius.com team.
 *
 * @author Svetoslav Marinov | http://orbisius.com
 * @package WebWeb_WP_MibewService
 */
class WebWeb_WP_MibewService {
    private $live = 0; // turn on when live!
    private $service_end_point = 'http://orbisius.com/service/kv/1.0/';
    private $dev_service_end_point = 'http://localhost/service/kv/1.0/';

    private $method = null;
    private $params = array();
    private $meta = array();
    private $data = array();

    /**
     * Setups the method to be called and
     */
    public function __construct($method, $params = array()) {
        $method = strtolower($method);
        $method = trim($method, ' .');

        // The format should be: wp.plugins.get_all as this API can be used by modile apps someday
        if (!preg_match('#^[a-z][\w-]+\.([a-z][\w-]+\.)?[a-z][\w-]+$#si', $method)) {
            throw new Exception('Invalid method.');
        }

        $this->live = empty($_SERVER['DEV_ENV']); // if it exists -> on dev machine.

        $this->method = $method;
        $this->params = $params;
    }

    /**
     * Calls the remote service.
     * It currently relies on caching from the calling method because
     * I am assumming the data will be saved into the db.
     * The result will contain data and meta section
     */
    public function call() {
        $browser = new WebWeb_WP_MibewCrawler();

        $end_point = $this->live ? $this->service_end_point : $this->dev_service_end_point;

        $url = $end_point . '?' . http_build_query(array('method' => $this->method, 'params' => $this->params));

        $this->status = 0;
        $result_buffer = array();

        if ($browser->fetch($url)) {
            $buffer = $browser->get_content();

            parse_str($buffer, $result_buffer);

            if (!empty($result_buffer['meta'])) {
                $this->meta = $result_buffer['meta'];
                $this->data = $result_buffer['data'];
                $this->status = 1;
            }
        }

        return $this->status;
    }

    /**
     * returns the meta that was previously filled out by a recent ->call
     * @param void
     * @return array
     */
    public function get_meta() {
        return $this->meta;
    }

    /**
     * returns the data that was previously filled out by a recent ->call
     * @param void
     * @return array
     */
    public function get_data() {
        return $this->data;
    }
}